# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/01m_utils.download.ipynb.

# %% auto 0
__all__ = ['download', 'download_and_extract', 'download_gdrive', 'download_and_extract_gdrive']

# %% ../../nbs/01m_utils.download.ipynb 2
from fastcore.all import *
import polvo as pv

# %% ../../nbs/01m_utils.download.ipynb 4
def download(url, save_dir, chunk_size=1024):
    """Download file from url"""
    import requests
    r = requests.get(url, stream=True)
    save_path = Path(save_dir)/Path(url).name
    with open(str(save_path), "wb") as f:
        bar_total = r.headers.get("Content-Length")
        bar = pv.pbar(unit="B", total=int(bar_total) if bar_total else None)
        for chunk in r.iter_content(chunk_size=chunk_size):
            if chunk:  # filter out keep-alive new chunks
                bar.update(len(chunk))
                f.write(chunk)
    return save_path

# %% ../../nbs/01m_utils.download.ipynb 5
def download_and_extract(url, save_dir, chunk_size=1024):
    save_path = download(url=str(url), save_dir=save_dir, chunk_size=chunk_size)
    import shutil; shutil.unpack_archive(filename=str(save_path), extract_dir=str(save_dir))
    return save_dir

# %% ../../nbs/01m_utils.download.ipynb 7
def download_gdrive(url):
    """Download from gdrive, passing virus scan for big files."""
    import gdown
    return gdown.download(url=str(url), quiet=False)

# %% ../../nbs/01m_utils.download.ipynb 8
def download_and_extract_gdrive(url, extract_dir):
    extract_dir = Path(extract_dir)
    extract_dir.mkdir(exist_ok=True, parents=True)

    filename = download_gdrive(url=url)
    import shutil; shutil.unpack_archive(filename=filename, extract_dir=str(extract_dir))



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/00g_test.ipynb.

# %% auto 0
__all__ = ['ROOT_DIR', 'SEG_IMAGE', 'SEG_MASK']

# %% ../nbs/00g_test.ipynb 2
import polvo as pv
from pathlib import Path

# %% ../nbs/00g_test.ipynb 3
ROOT_DIR =  Path(pv.__file__).parent.parent/'assets'
SEG_IMAGE = ROOT_DIR/'segmentation/image1.png'
SEG_MASK = ROOT_DIR/'segmentation/mask1.png'



# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/02f_datasets.download.ipynb.

# %% auto 0
__all__ = ['snacks_tiny']

# %% ../../nbs/02f_datasets.download.ipynb 2
import polvo as pv

# %% ../../nbs/02f_datasets.download.ipynb 3
# TODO: This just goes to tmp folder
def snacks_tiny():
    data_dir = pv.mkdir('snacks', tmp=True, exist_ok=True, overwrite=True)
    data_dir = pv.download_and_extract('https://github.com/lgvaz/polvo-datasets-hub/releases/download/snacks/snacks.zip',
                                       data_dir)
    return data_dir



# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/01t_utils.misc.ipynb.

# %% auto 0
__all__ = ['kwargs_grid', 'safe_zip', 'skip_error']

# %% ../../nbs/01t_utils.misc.ipynb 2
from fastcore.all import *
import numbers
import polvo as pv

from PIL import Image

# %% ../../nbs/01t_utils.misc.ipynb 3
def kwargs_grid(**kwargs):
    "Returns a generator with all combinations of kwargs"
    return (dict(zip(kwargs.keys(), v)) for v in itertools.product(*kwargs.values()))

# %% ../../nbs/01t_utils.misc.ipynb 5
@functools.wraps(zip)
def safe_zip(*args, **kwargs):
    if len(set(map(len, args))) not in (0, 1):
        raise ValueError(f'All elements should have the same size, but got {[len(x) for x in args]}')
    return zip(*args, **kwargs)

# %% ../../nbs/01t_utils.misc.ipynb 8
class skip_error:
    def __init__(self, fn, log=True):
        'Returns the error instead of raising it.'
        store_attr()
        
    def __call__(self, *args, **kwargs):
        try: return self.fn(*args, **kwargs)
        except Exception as e: 
            if self.log: print((args, kwargs))
            return e



# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/01c_utils.script.ipynb.

# %% auto 0
__all__ = ['call_parse', 'test_script']

# %% ../../nbs/01c_utils.script.ipynb 2
from fastcore.all import *
import inspect

# %% ../../nbs/01c_utils.script.ipynb 3
def call_parse(func=None, nested=False):
    """Decorator to create a simple CLI from `func` using `anno_parser`
    Copied from: https://github.com/fastai/fastcore/blob/master/fastcore/script.py with a single change (marked by comment)
    """
    if func is None: return partial(call_parse, nested=nested)

    @functools.wraps(func)
    def _f(*args, **kwargs):
        mod = inspect.getmodule(inspect.currentframe().f_back)
        if not mod: return func(*args, **kwargs)
        if not SCRIPT_INFO.func and mod.__name__=="__main__": SCRIPT_INFO.func = func.__name__
        if len(sys.argv)>1 and sys.argv[1]=='': sys.argv.pop(1)
        p = anno_parser(func)
        if nested: args, sys.argv[1:] = p.parse_known_args()
        else: args = p.parse_args()
        args = args.__dict__
        xtra = otherwise(args.pop('xtra', ''), eq(1), p.prog)
        tfunc = trace(func) if args.pop('pdb', False) else func
        # CHANGED: Uses `print` instead of `return`, so output can be piped in terminal commands
        print(tfunc(**merge(args, args_from_prog(func, xtra)))) 

    mod = inspect.getmodule(inspect.currentframe().f_back)
    if getattr(mod, '__name__', '') =="__main__":
        setattr(mod, func.__name__, _f)
        SCRIPT_INFO.func = func.__name__
        return _f()
        
    else: return _f

# %% ../../nbs/01c_utils.script.ipynb 4
@call_parse
def test_script(msg:Param("The message", str),
                upper:Param("Convert to uppercase?", store_true)=False):
    "Print `msg`, optionally converting to uppercase"
    return msg.upper() if upper else msg



# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/01p_utils.visualization.ipynb.

# %% auto 0
__all__ = ['show_image', 'get_image_grid', 'show_grid', 'normalize_axes', 'plot_grid', 'image_grid', 'grid_from_sequence',
           'image_size', 'image_sizes_hist']

# %% ../../nbs/01p_utils.visualization.ipynb 3
from fastcore.all import *
import random
import polvo as pv
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
from mpl_toolkits.axes_grid1 import ImageGrid

# %% ../../nbs/01p_utils.visualization.ipynb 4
@delegates(plt.subplots)
def show_image(image, ax=None, cmap=None, show:bool=False, **kwargs):
    if ax is None: fig, ax = plt.subplots(**kwargs)
    ax.imshow(image, cmap=cmap)
    ax.set_axis_off()
    if show: plt.show()
    return ax

# %% ../../nbs/01p_utils.visualization.ipynb 8
@delegates(ImageGrid)
def get_image_grid(
    nitems=None,
    ncols=3,
    nrows=None,
    pad=0.0,
    figsize=None,
    **kwargs
):
    "Creates axes optimized for displaying images."
    nrows = nrows or int(np.ceil(nitems / ncols))
    figsize = figsize or (14, 4 * nrows)

    fig = plt.figure(figsize=figsize)
    grid = ImageGrid(fig, 111, (nrows, ncols), axes_pad=pad, **kwargs)
    
    return fig, grid

# %% ../../nbs/01p_utils.visualization.ipynb 9
def show_grid(grid, shows, show=True):
    for ax, show in zip(grid, shows): show(ax=ax)
    if show: plt.show()

# %% ../../nbs/01p_utils.visualization.ipynb 10
def normalize_axes(grid, xmax, ymax):
    "Expand all axes to have the same dimensions."
    for ax in grid:
        ax.set_xlim(right=xmax)
        ax.set_ylim(bottom=ymax)

# %% ../../nbs/01p_utils.visualization.ipynb 11
@delegates(get_image_grid, but=['nitems'])
def plot_grid(shows, show=True, xmax=None, ymax=None, **kwargs):
    "Quickly plot a grid of images."
    fig, grid = get_image_grid(nitems=len(shows), **kwargs)
    if xmax or ymax: normalize_axes(grid, xmax, ymax)
    show_grid(grid, shows, show=show)
    return fig, grid

# %% ../../nbs/01p_utils.visualization.ipynb 12
@delegates(plot_grid, but=['shows'])
def image_grid(images, *args, **kwargs):
    shows = [partial(show_image, o) for o in images]
    return plot_grid(shows, *args, **kwargs)

# %% ../../nbs/01p_utils.visualization.ipynb 14
@delegates(image_grid)
def grid_from_sequence(sequence, get_image, nitems=9, idxs=None, **kwargs):
    "Generates a grid of images from a subset of items from the sequence."
    idxs = idxs or random.sample(range(0, len(sequence)), nitems)
    images = []
    for idx in idxs:
        item = sequence[idx]
        images.append(get_image(item))
    
    return image_grid(images, **kwargs)

# %% ../../nbs/01p_utils.visualization.ipynb 16
def image_size(image_file):
    with pv.open_image(image_file) as image:
        return image.size

# %% ../../nbs/01p_utils.visualization.ipynb 17
def image_sizes_hist(
    image_files # Sequence of image filepaths.
):
    "Plot a histogram of widths and heights."
    sizes = parallel(image_size, image_files, progress=pv.pbar)
    widths, heights = zip(*sizes)
    plt.hist(widths, label='width')
    plt.hist(heights, label='heights')
    plt.legend()



# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/01e_utils.io.ipynb.

# %% auto 0
__all__ = ['image_extensions', 'save_txt', 'open_txt', 'open_json', 'save_json', 'mkdir', 'extract_files', 'get_files',
           'get_image_files', 'open_image', 'open_mask', 'save_image', 'RenderDict']

# %% ../../nbs/01e_utils.io.ipynb 2
import shutil, tempfile
import numpy as np
import polvo as pv
import uuid, json, mimetypes
from pathlib import Path
from fastcore.all import *
from PIL import Image, ImageOps

# %% ../../nbs/01e_utils.io.ipynb 3
def save_txt(s, path, append=False):
    mode = 'a' if append else 'w'
    with open(str(path), mode) as f: f.write(s)

# %% ../../nbs/01e_utils.io.ipynb 4
def open_txt(path):
    with open(str(path), 'r') as f: return f.read()
#     return np.loadtxt(path, dtype=str, delimiter="\n").tolist()

# %% ../../nbs/01e_utils.io.ipynb 6
def open_json(path):
    with open(str(path)) as f: return json.load(f)

# %% ../../nbs/01e_utils.io.ipynb 7
def save_json(d, path):
    with open(str(path), 'w') as f: json.dump(d, f)

# %% ../../nbs/01e_utils.io.ipynb 9
def mkdir(path, exist_ok=False, parents=False, overwrite=False, tmp=False, **kwargs):
    "Creates and returns a directory defined by `path`, optionally removing previous existing directory if `overwrite` is `True`"
    path = Path(path)
    if tmp:
        path = tempfile.gettempdir() / path
        parents = True
    if path.exists() and overwrite: shutil.rmtree(path)
    path.mkdir(exist_ok=exist_ok, parents=parents, **kwargs)
    return path

# %% ../../nbs/01e_utils.io.ipynb 11
def extract_files(files, extract_to_dir, show_pbar=True):
    for file in pbar(files, show=show_pbar):
        extract_path = extract_to_dir / Path(file).with_suffix("").name
        shutil.unpack_archive(file, extract_path)
    return extract_to_dir

# %% ../../nbs/01e_utils.io.ipynb 12
def _get_files(p, fs, extensions=None):
    "COPIED FROM https://github.com/fastai/fastai/blob/master/nbs/05_data.transforms.ipynb"
    p = Path(p)
    res = [p/f for f in fs if not f.startswith('.')
           and ((not extensions) or f'.{f.split(".")[-1].lower()}' in extensions)]
    return res

# %% ../../nbs/01e_utils.io.ipynb 13
def get_files(path, extensions=None, recurse=True, folders=None, followlinks=True):
    """Get all the files in `path` with optional `extensions`, optionally with `recurse`, only in `folders`, if specified.
    COPIED FROM https://github.com/fastai/fastai/blob/master/nbs/05_data.transforms.ipynb
    """
    path = Path(path)
    folders=L(folders)
    extensions = setify(extensions)
    extensions = {e.lower() for e in extensions}
    if recurse:
        res = []
        for i,(p,d,f) in enumerate(os.walk(path, followlinks=followlinks)): # returns (dirpath, dirnames, filenames)
            if len(folders) !=0 and i==0: d[:] = [o for o in d if o in folders]
            else:                         d[:] = [o for o in d if not o.startswith('.')]
            if len(folders) !=0 and i==0 and '.' not in folders: continue
            res += _get_files(p, f, extensions)
    else:
        f = [o.name for o in os.scandir(path) if o.is_file()]
        res = _get_files(path, f, extensions)
    return L(res)

# %% ../../nbs/01e_utils.io.ipynb 15
image_extensions = set(k for k,v in mimetypes.types_map.items() if v.startswith('image/'))

# %% ../../nbs/01e_utils.io.ipynb 16
def get_image_files(path, recurse=True, folders=None):
    """Get image files in `path` recursively, only in `folders`, if specified.
    COPIED FROM https://github.com/fastai/fastai/blob/master/nbs/05_data.transforms.ipynb
    """
    return get_files(path, extensions=image_extensions, recurse=recurse, folders=folders)

# %% ../../nbs/01e_utils.io.ipynb 18
def open_image(path, gray=False, ignore_exif=True) -> Image.Image:
    "Open an image from disk `path` as a PIL Image"
    color = "L" if gray else "RGB"
    image = Image.open(str(path))
    if not ignore_exif: image = ImageOps.exif_transpose(image)
    return image.convert(color)

# %% ../../nbs/01e_utils.io.ipynb 20
@delegates(open_image)
def open_mask(path, gray=True, **kwargs) -> Image.Image:
    return open_image(path, gray=gray, **kwargs)

# %% ../../nbs/01e_utils.io.ipynb 22
def save_image(image, path):
    if isinstance(image, np.ndarray): image = Image.fromarray(image)
    return image.save(str(path))

# %% ../../nbs/01e_utils.io.ipynb 24
class RenderDict:
    "From https://www.reddit.com/r/IPython/comments/34t4m7/lpt_print_json_in_collapsible_format_in_ipython/"
    def __init__(self, json_data):
        if isinstance(json_data, dict):
            self.json_str = json.dumps(json_data)
        else:
            self.json_str = json_data
        self.uuid = str(uuid.uuid4())

    def _ipython_display_(self):
        from IPython.display import display_javascript, display_html, display
        display_html('<div id="{}" style="height: 600px; width:100%;"></div>'.format(self.uuid), raw=True)
        display_javascript("""
        require(["https://rawgit.com/caldwell/renderjson/master/renderjson.js"], function() {
        document.getElementById('%s').appendChild(renderjson(%s))
        });
        """ % (self.uuid, self.json_str), raw=True)



# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/05a_classification.explorer.ipynb.

# %% auto 0
__all__ = ['DataExplorer']

# %% ../../nbs/05a_classification.explorer.ipynb 2
#TODO: optimize imports
from fastcore.all import *

import numpy as np
import pandas as pd
import polvo as pv

from PIL import Image

# %% ../../nbs/05a_classification.explorer.ipynb 7
class DataExplorer:
    def __init__(self, label_ids, idx2image, id2label, label2id):
        store_attr()
        self.grouped_y = self._group_y()
        
    def _group_y(self):
        label_df = pd.DataFrame(self.label_ids, columns=["label_id"])
        label_df['label'] = label_df['label_id'].map(self.id2label) #
        label_df["item_idx"] = label_df.index
        
        label_df = label_df.groupby("label_id").agg({"label": "first", "item_idx": list}).reset_index()
        label_df["n_samples"] = label_df.item_idx.str.len()
        label_df = label_df.sort_values("n_samples", ascending=False).reset_index(drop=True)
        return label_df

    def most_common(self):
        "Return a dataframe ordered by most common classes."
        return self.grouped_y.copy()

    def show_label_id(self, id, slice_=slice(None)):
        # TODO: can we also pick the class name to show?
        idx = self.grouped_y[self.grouped_y["label_id"] == id].index[0]
        return self.show_most_common(idx=idx, slice_=slice_)
    
    def show_label(self, label, slice_=slice(None)):
        return self.show_label_id(self.label2id(label), slice_=slice_) #

    def show_most_common(self, idx: int, slice_=slice(None)):
        item_idxs = self.grouped_y.iloc[idx].item_idx
        item_idxs = item_idxs[slice_]
        
        images = [self.idx2image(i) for i in item_idxs]
        xmax, ymax = np.max([image.size for image in images], axis=0)
        
        return pv.image_grid(images, xmax=xmax, ymax=ymax)



# Autogenerated by nbdev

d = { 'settings': { 'branch': 'master',
                'doc_baseurl': '/polvo/',
                'doc_host': 'https://lgvaz.github.io',
                'git_url': 'https://github.com/lgvaz/polvo/',
                'lib_path': 'polvo'},
  'syms': { 'polvo.classification.baseline.baseline': { 'polvo.classification.baseline.baseline.FastaiMetricWrapper': ( 'classification.baseline.baseline.html#fastaimetricwrapper',
                                                                                                                        'polvo/classification/baseline/baseline.py'),
                                                        'polvo.classification.baseline.baseline.FastaiMetricWrapper.__init__': ( 'classification.baseline.baseline.html#fastaimetricwrapper.__init__',
                                                                                                                                 'polvo/classification/baseline/baseline.py'),
                                                        'polvo.classification.baseline.baseline.FastaiMetricWrapper.accumulate': ( 'classification.baseline.baseline.html#fastaimetricwrapper.accumulate',
                                                                                                                                   'polvo/classification/baseline/baseline.py'),
                                                        'polvo.classification.baseline.baseline.calculate_metrics': ( 'classification.baseline.baseline.html#calculate_metrics',
                                                                                                                      'polvo/classification/baseline/baseline.py'),
                                                        'polvo.classification.baseline.baseline.fixed_probs': ( 'classification.baseline.baseline.html#fixed_probs',
                                                                                                                'polvo/classification/baseline/baseline.py'),
                                                        'polvo.classification.baseline.baseline.random_probs': ( 'classification.baseline.baseline.html#random_probs',
                                                                                                                 'polvo/classification/baseline/baseline.py')},
            'polvo.classification.explorer': { 'polvo.classification.explorer.DataExplorer': ( 'classification.explorer.html#dataexplorer',
                                                                                               'polvo/classification/explorer.py'),
                                               'polvo.classification.explorer.DataExplorer.__init__': ( 'classification.explorer.html#dataexplorer.__init__',
                                                                                                        'polvo/classification/explorer.py'),
                                               'polvo.classification.explorer.DataExplorer._group_y': ( 'classification.explorer.html#dataexplorer._group_y',
                                                                                                        'polvo/classification/explorer.py'),
                                               'polvo.classification.explorer.DataExplorer.most_common': ( 'classification.explorer.html#dataexplorer.most_common',
                                                                                                           'polvo/classification/explorer.py'),
                                               'polvo.classification.explorer.DataExplorer.show_label': ( 'classification.explorer.html#dataexplorer.show_label',
                                                                                                          'polvo/classification/explorer.py'),
                                               'polvo.classification.explorer.DataExplorer.show_label_id': ( 'classification.explorer.html#dataexplorer.show_label_id',
                                                                                                             'polvo/classification/explorer.py'),
                                               'polvo.classification.explorer.DataExplorer.show_most_common': ( 'classification.explorer.html#dataexplorer.show_most_common',
                                                                                                                'polvo/classification/explorer.py')},
            'polvo.datasets.download': { 'polvo.datasets.download.snacks_tiny': ( 'datasets.download.html#snacks_tiny',
                                                                                  'polvo/datasets/download.py')},
            'polvo.segmentation.vis': { 'polvo.segmentation.vis.overlay_mask': ( 'segmentation.vis.html#overlay_mask',
                                                                                 'polvo/segmentation/vis.py')},
            'polvo.test': {},
            'polvo.utils.download': { 'polvo.utils.download.download': ('utils.download.html#download', 'polvo/utils/download.py'),
                                      'polvo.utils.download.download_and_extract': ( 'utils.download.html#download_and_extract',
                                                                                     'polvo/utils/download.py'),
                                      'polvo.utils.download.download_and_extract_gdrive': ( 'utils.download.html#download_and_extract_gdrive',
                                                                                            'polvo/utils/download.py'),
                                      'polvo.utils.download.download_gdrive': ( 'utils.download.html#download_gdrive',
                                                                                'polvo/utils/download.py')},
            'polvo.utils.io': { 'polvo.utils.io.RenderDict': ('utils.io.html#renderdict', 'polvo/utils/io.py'),
                                'polvo.utils.io.RenderDict.__init__': ('utils.io.html#renderdict.__init__', 'polvo/utils/io.py'),
                                'polvo.utils.io.RenderDict._ipython_display_': ( 'utils.io.html#renderdict._ipython_display_',
                                                                                 'polvo/utils/io.py'),
                                'polvo.utils.io._get_files': ('utils.io.html#_get_files', 'polvo/utils/io.py'),
                                'polvo.utils.io.extract_files': ('utils.io.html#extract_files', 'polvo/utils/io.py'),
                                'polvo.utils.io.get_files': ('utils.io.html#get_files', 'polvo/utils/io.py'),
                                'polvo.utils.io.get_image_files': ('utils.io.html#get_image_files', 'polvo/utils/io.py'),
                                'polvo.utils.io.mkdir': ('utils.io.html#mkdir', 'polvo/utils/io.py'),
                                'polvo.utils.io.open_image': ('utils.io.html#open_image', 'polvo/utils/io.py'),
                                'polvo.utils.io.open_json': ('utils.io.html#open_json', 'polvo/utils/io.py'),
                                'polvo.utils.io.open_mask': ('utils.io.html#open_mask', 'polvo/utils/io.py'),
                                'polvo.utils.io.open_txt': ('utils.io.html#open_txt', 'polvo/utils/io.py'),
                                'polvo.utils.io.save_image': ('utils.io.html#save_image', 'polvo/utils/io.py'),
                                'polvo.utils.io.save_json': ('utils.io.html#save_json', 'polvo/utils/io.py'),
                                'polvo.utils.io.save_txt': ('utils.io.html#save_txt', 'polvo/utils/io.py')},
            'polvo.utils.logging': {'polvo.utils.logging.pbar': ('utils.logging.html#pbar', 'polvo/utils/logging.py')},
            'polvo.utils.misc': { 'polvo.utils.misc.kwargs_grid': ('utils.misc.html#kwargs_grid', 'polvo/utils/misc.py'),
                                  'polvo.utils.misc.safe_zip': ('utils.misc.html#safe_zip', 'polvo/utils/misc.py'),
                                  'polvo.utils.misc.skip_error': ('utils.misc.html#skip_error', 'polvo/utils/misc.py'),
                                  'polvo.utils.misc.skip_error.__call__': ('utils.misc.html#skip_error.__call__', 'polvo/utils/misc.py'),
                                  'polvo.utils.misc.skip_error.__init__': ('utils.misc.html#skip_error.__init__', 'polvo/utils/misc.py')},
            'polvo.utils.script': { 'polvo.utils.script.call_parse': ('utils.script.html#call_parse', 'polvo/utils/script.py'),
                                    'polvo.utils.script.test_script': ('utils.script.html#test_script', 'polvo/utils/script.py')},
            'polvo.utils.visualization': { 'polvo.utils.visualization.get_image_grid': ( 'utils.visualization.html#get_image_grid',
                                                                                         'polvo/utils/visualization.py'),
                                           'polvo.utils.visualization.grid_from_sequence': ( 'utils.visualization.html#grid_from_sequence',
                                                                                             'polvo/utils/visualization.py'),
                                           'polvo.utils.visualization.image_grid': ( 'utils.visualization.html#image_grid',
                                                                                     'polvo/utils/visualization.py'),
                                           'polvo.utils.visualization.image_size': ( 'utils.visualization.html#image_size',
                                                                                     'polvo/utils/visualization.py'),
                                           'polvo.utils.visualization.image_sizes_hist': ( 'utils.visualization.html#image_sizes_hist',
                                                                                           'polvo/utils/visualization.py'),
                                           'polvo.utils.visualization.normalize_axes': ( 'utils.visualization.html#normalize_axes',
                                                                                         'polvo/utils/visualization.py'),
                                           'polvo.utils.visualization.plot_grid': ( 'utils.visualization.html#plot_grid',
                                                                                    'polvo/utils/visualization.py'),
                                           'polvo.utils.visualization.show_grid': ( 'utils.visualization.html#show_grid',
                                                                                    'polvo/utils/visualization.py'),
                                           'polvo.utils.visualization.show_image': ( 'utils.visualization.html#show_image',
                                                                                     'polvo/utils/visualization.py')}}}



# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/08a_segmentation.vis.ipynb.

# %% auto 0
__all__ = ['overlay_mask']

# %% ../../nbs/08a_segmentation.vis.ipynb 3
import polvo as pv
import PIL
import numpy as np
import matplotlib as mpl

# %% ../../nbs/08a_segmentation.vis.ipynb 4
def overlay_mask(
    image:PIL.Image,
    mask:PIL.Image,
    nclasses:int,
    cmap:mpl.colors.Colormap=mpl.colormaps['gist_ncar'],
    alpha:float=0.5,
):
    mask = np.asarray(mask)
    if not len(mask.shape) == 2: raise ValueError(f'Mask bust be 2d, but got {mask.shape}')
    
    colored_mask = np.zeros((*mask.shape, 3), dtype=np.uint8)
    for class_idx in np.unique(mask):
        mask_idxs = mask == class_idx
        colored_mask[mask_idxs] = np.array(cmap(class_idx/nclasses)[:3]) * 255

    return PIL.Image.blend(image, PIL.Image.fromarray(colored_mask), alpha)



# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/01b_utils.logging.ipynb.

# %% auto 0
__all__ = ['pbar']

# %% ../../nbs/01b_utils.logging.ipynb 2
from time import sleep
from tqdm.autonotebook import tqdm

# %% ../../nbs/01b_utils.logging.ipynb 3
def pbar(*args, show=True, total=None, **kwargs): 
    return tqdm(*args, total=total, **kwargs) if show else iter



# AUTOGENERATED! DO NOT EDIT! File to edit: ../../../nbs/02b_classification.baseline.baseline.ipynb.

# %% auto 0
__all__ = ['FastaiMetricWrapper', 'calculate_metrics', 'fixed_probs', 'random_probs']

# %% ../../../nbs/02b_classification.baseline.baseline.ipynb 1
from fastai.vision.all import *
from torch.utils.data import TensorDataset

# %% ../../../nbs/02b_classification.baseline.baseline.ipynb 3
class FastaiMetricWrapper(GetAttr):
    _default = 'metric'
    def __init__(self, metric):
        self.metric = metric
        
    def accumulate(self, pred, yb):
        fake_learner = namedtuple('FakeLearner', 'pred yb y to_detach')(pred, (yb,), yb, to_detach)
        self.metric.accumulate(fake_learner)

# %% ../../../nbs/02b_classification.baseline.baseline.ipynb 4
def calculate_metrics(dl, metrics, probs_func, n_classes=None):
    "Calculate metrics for a baseline method `probs_func`"
    n_classes = n_classes or get_c(dl)
    if n_classes == 0:
        raise ValueError("Couldn't automatically infer number of classes from dataloader. Please explicitly specify it with `n_classes`")
        
    metrics = L(metrics).map(compose(mk_metric, FastaiMetricWrapper))
    for metric in metrics: metric.reset()

    for x,y in dl:
        pred = probs_func(x, y, n_classes)
        for metric in metrics: metric.accumulate(pred, y)

    return {metric.name: metric.value for metric in metrics}

# %% ../../../nbs/02b_classification.baseline.baseline.ipynb 8
def fixed_probs(x, y, n_classes, class_id):
    pred = torch.zeros(find_bs(y), n_classes).to(y.device)
    pred[:, class_id] = 1
    return pred

# %% ../../../nbs/02b_classification.baseline.baseline.ipynb 11
def random_probs(x, y, n_classes):
    pred = torch.randn(find_bs(y), n_classes).to(y.device)
    pred = torch.nn.functional.softmax(pred, dim=-1)
    return pred
